/*
 *
 * Some conn funcs prototypes
 *
 *
 */
#ifndef CONN_H
#define CONN_H

#include <unistd.h>
#include <linux/atm.h>
#include <sys/types.h>

#include "af_lane.h"

void conn_call_callbacks(void);
void conn_get_fds(fd_set *fds);
void conn_get_connecting_fds(fd_set *fds);
int conn_check_incoming(fd_set *fds);
int conn_create_listensocket(HANDLE sap_context, HANDLE conn_context,
			     unsigned short blli_codepoint,
			     const char *qos_spec,
			     HANDLE *p_conn_handle);
int conn_set_kernel_socket(int fd);
/*
void conn_convert_to_data_vcc(HANDLE conn_handle);
*/
#endif /* CONN_H */
/*
 *
 * $Log: conn.h,v $
 * Revision 1.2  1996/08/06 14:14:11  carnil
 * Cleaning up
 * Address handling moved to address.c from conn.c
 *
 * Revision 1.1  1996/05/23 11:50:54  carnil
 * Initial revision
 *
 *
 */
