/* sap.h - SAP manipulations */

/* Written 1996,1997 by Werner Almesberger, EPFL-LRC */


#ifndef SAP_H
#define SAP_H

#include <linux/atmsvc.h>

#include "atmsap.h"


int sap_compat(const struct sockaddr_atmsvc *old_addr,
  const struct sockaddr_atmsvc *new_addr,struct sockaddr_atmsvc *res_addr,
  const struct atm_sap *old_sap,const struct atm_sap *new_sap,
  struct atm_sap *res_sap,const struct atm_qos *old_qos,
  const struct atm_qos *new_qos,struct atm_qos *res_qos);
int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *addr,
  const struct atm_sap *sap,const struct atm_qos *qos);
void sap_decode(Q_DSC *dsc,struct sockaddr_atmsvc *addr,struct atm_sap *sap,
  struct atm_qos *qos);

#endif
