/*
 * Marko Kiiskila carnil@cs.tut.fi 
 * 
 * Copyright (c) 1996
 * Tampere University of Technology - Telecommunications Laboratory
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation is hereby granted,
 * provided that both the copyright notice and this
 * permission notice appear in all copies of the software,
 * derivative works or modified versions, and any portions
 * thereof, that both notices appear in supporting
 * documentation, and that the use of this software is
 * acknowledged in any publications resulting from using
 * the software.
 * 
 * TUT ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION AND DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS
 * SOFTWARE.
 * 
 */
/*
 * Include file for timer funcs not used in DIGITAL's code
 *
 * $Id: timers.h,v 1.3 1996/08/06 14:14:11 carnil Exp carnil $
 *
 */
/* Global includes */
#include <sys/time.h>

/* Digital includes */
#include "g_types.h"

void timer_call_callback(HANDLE timer_handle);
struct timeval* timer_get_expiration(HANDLE timer_handle);
HANDLE timer_find_soonest(void);
void timer_usage(void);
/*
 *
 * $Log: timers.h,v $
 * Revision 1.3  1996/08/06 14:14:11  carnil
 * Cleaning up
 *
 * Revision 1.2  1996/04/25 19:42:13  carnil
 * Copyright notice
 *
 * Revision 1.1  1996/01/19 06:21:43  carnil
 * Initial revision
 *
 *
 */
