/* isp.h - Internal Signaling Protocol test generator */

/* Written 1997 by Werner Almesberger, EPFL-ICA */


#ifndef ISP_H
#define ISP_H

#include <linux/atmsvc.h>


void send_msg(const struct atmsvc_msg *msg);
void recv_msg(struct atmsvc_msg *msg);
void dump_msg(const struct atmsvc_msg *msg);
int str2errno(const char *str);

#endif
