/* io.h - I/O operations */
 
/* Written 1996 by Werner Almesberger, EPFL-LRC */
 

#ifndef IO_H
#define IO_H

#include <sys/socket.h> /* for struct sockaddr */
#include <linux/atm.h> /* for struct sockaddr_atmsvc */


void open_all(void);
void close_all(void);
void poll_loop(void);

#endif
